#!/bin/ksh

### PharmAnnot PIPELINE ###
## merge_GATK.sh
## Version : 1.0.0
## Description : This script merge two vcf file
## Usage : qsub -pe smp <nb thread> -v INPUTFILE1=</path/to/the/input/vcf/file>,INPUTFILE2=</path/to/the/input/vcf/file>,REF=</path/to/the/ref/file>,OUTPUTFILE=</path/to/the/output/vcf/file>,[LOGFILE=/path/to/the/log/file],[CONFIGFILE=/path/to/the/config/file] merge_GATK.sh
## Output : .vcf file containing annotations
## Requirements : jdk 1.8.0

## Author : simon.verdez@chu-dijon.fr
## Creation Date : 20180702
## last revision date : 20180702
## Known bugs : None

#$ -q batch
#$ -V



# Log file path option
if [ -z ${LOGFILE} ]
then
    LOGFILE=merge_GATK.$(date +"%F_%H-%M-%S").log
fi

# Config file path option
if [ -z ${CONFIGFILE} ]
then
    CONFIGFILE=analysis_config.tsv
fi

# Logging
exec 1>> $LOGFILE 2>&1
echo "$(date +"%F_%H-%M-%S"): START"

# Check if config file exist
if [ ! -f $CONFIGFILE ]
then
    echo "Config file does not exist"
    echo "$(date +"%F_%H-%M-%S"): END"
    touch merge_GATK.failed
    exit 1
fi

# Check if input1 file exist
if [ ! -f $INPUTFILE1 ]
then
    echo "Input1 file does not exist"
    echo "$(date +"%F_%H-%M-%S"): END"
    touch merge_GATK.failed
    exit 1
fi

# Check if input2 file exist
if [ ! -f $INPUTFILE2 ]
then
    echo "Input2 file does not exist"
    echo "$(date +"%F_%H-%M-%S"): END"
    touch merge_GATK.failed
    exit 1
fi

# Check if ref file exist
if [ ! -f $REF ]
then
    echo "ref file does not exist"
    echo "$(date +"%F_%H-%M-%S"): END"
    touch merge_GATK.failed
    exit 1
fi

# Check if output file is specified
if [ -z $OUTPUTFILE ]
then
    echo "Output file is not specified"
    echo "$(date +"%F_%H-%M-%S"): END"
    touch merge_GATK.failed
    exit 1
fi

JAVA=$(grep "javacmd" $CONFIGFILE | awk '{print $2}')
PATHSCRIPT=$(grep "pipelinebase" $CONFIGFILE | awk '{print $2}')
#GATK=$(grep "GATKbase" $CONFIGFILE | awk '{print $2}')

echo "command : $JAVA -jar /work/gad/shared/bin/GATK_3.8/GenomeAnalysisTK.jar -T CombineVariants -R $REF --variant $INPUTFILE1 --variant $INPUTFILE2 -o merge.vcf -genotypeMergeOptions UNIQUIFY"
$JAVA -jar /work/gad/shared/bin/GATK_3.8/GenomeAnalysisTK.jar -T CombineVariants -R $REF --variant $INPUTFILE2 --variant $INPUTFILE2 -genotypeMergeOptions UNIQUIFY > $OUTPUTFILE

genotypeMerge_exitcode=$?
echo "GenotypeMerge exit code : $genotypeMerge_exitcode"
if [ $genotypeMerge_exitcode != 0 ]
then
    echo "$(date +"%F_%H-%M-%S"): END"
    touch merge.failed
    exit 1
fi

echo "$(date +"%F_%H-%M-%S"): END"
